@@ -4,6 +4,8 @@ from .t2t import cache_ontology_set
 from .t2t import cache_ontology
 from .t2t import cache_exists
 from .t2t import clear_cache
+from .t2t import map_tagged_terms
 from .mapper import Mapper
 from .preprocess import preprocess_file
-from .preprocess import preprocess_terms
\ No newline at end of file
+from .preprocess import preprocess_terms
+from .preprocess import preprocess_tagged_terms
\ No newline at end of file
