@@ -114,7 +114,7 @@ def parse_list_file(file_path):
 
 
 def parse_csv_file(file_path, term_column_name, term_id_column_name, separator=','):
-    data = pd.read_csv(file_path, sep=separator)
+    data = pd.read_csv(file_path, sep=separator, engine='python')
     if term_column_name not in data.columns:
         raise ValueError("Could not find specified column name for input terms: " + term_column_name)
     terms = data[term_column_name].values
